/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 *
 *
 *  This File was modified by Markus Fruehwirth.
 *  
 *  Purpose: Dump both Channels of PCM and FRQ Data to files
 *  Just activate plugin and find 4 new files in /tmp
 * 
 *  Installation: Copy this directory to XMMS Source directory
 *  and run there "make" and "make install"
 *
 *  25.09.2000
 */

#include "config.h"

#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "xmms/plugin.h"
#include "libxmms/util.h"
#include "xmms_logo.xpm"

typedef struct timeval
{
  long tv_sec;			/* seconds */
  long tv_usec;			/* microseconds */
}
timeval;

#define NUM_BANDS 16

static GtkWidget *window = NULL, *area;
static GdkPixmap *bg_pixmap = NULL, *draw_pixmap = NULL, *bar = NULL;
static GdkGC *gc = NULL;
static gint16 bar_heights[NUM_BANDS];
static gint timeout_tag;
static gdouble scale;
static FILE *li_frq = NULL, *re_frq = NULL, *li_pcm = NULL, *re_pcm = NULL;
static gint discrun, discrun_pcm;
static int counter;
static timeval startsong, startsongF;
static long savenow;

static void sanalyzer_init (void);
static void sanalyzer_cleanup (void);
static void sanalyzer_playback_start (void);
static void sanalyzer_playback_stop (void);
static void sanalyzer_render_freq (gint16 data[2][256]);
static void sanalyzer_render_pcm (gint16 data[2][512]);

VisPlugin sanalyzer_vp = {
  NULL,
  NULL,
  0,
  "Dump File Data" VERSION,
  2,
  2,
  sanalyzer_init,		/* init */
  sanalyzer_cleanup,		/* cleanup */
  NULL,				/* about */
  NULL,				/* configure */
  NULL,				/* disable_plugin */
  sanalyzer_playback_start,	/* playback_start */
  sanalyzer_playback_stop,	/* playback_stop */
  sanalyzer_render_pcm,		/* render_pcm */
  sanalyzer_render_freq		/* render_freq */
};

VisPlugin *
get_vplugin_info (void)
{
  return &sanalyzer_vp;
}

#define WIDTH 250
#define HEIGHT 100


static void
sanalyzer_destroy_cb (GtkWidget * w, gpointer data)
{
  sanalyzer_vp.disable_plugin (&sanalyzer_vp);
}

static void
sanalyzer_init (void)
{
  GdkColor color;
  int i;

  counter = 0;
  if (window)
    return;
  window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (window), "Dump 2 File");
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);
  gtk_widget_realize (window);
  bg_pixmap =
    gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL,
				  sanalyzer_xmms_logo_xpm);
  gdk_window_set_back_pixmap (window->window, bg_pixmap, 0);
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (sanalyzer_destroy_cb), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (gtk_widget_destroyed), &window);
  gtk_widget_set_usize (window, WIDTH, HEIGHT);
  gc = gdk_gc_new (window->window);
  draw_pixmap =
    gdk_pixmap_new (window->window, WIDTH, HEIGHT,
		    gdk_rgb_get_visual ()->depth);

  bar =
    gdk_pixmap_new (window->window, 25, HEIGHT, gdk_rgb_get_visual ()->depth);
  for (i = 0; i < HEIGHT / 2; i++)
    {
      color.red = 0xFFFF;
      color.green = ((i * 255) / (HEIGHT / 2)) << 8;
      color.blue = 0;

      gdk_color_alloc (gdk_colormap_get_system (), &color);
      gdk_gc_set_foreground (gc, &color);
      gdk_draw_line (bar, gc, 0, i, 24, i);
    }
  for (i = 0; i < HEIGHT / 2; i++)
    {
      color.red = (255 - ((i * 255) / (HEIGHT / 2))) << 8;
      color.green = 0xFFFF;
      color.blue = 0;

      gdk_color_alloc (gdk_colormap_get_system (), &color);
      gdk_gc_set_foreground (gc, &color);
      gdk_draw_line (bar, gc, 0, i + (HEIGHT / 2), 24, i + (HEIGHT / 2));
    }
  scale = HEIGHT / log (256);
  gdk_color_black (gdk_colormap_get_system (), &color);
  gdk_gc_set_foreground (gc, &color);

  area = gtk_drawing_area_new ();
  gtk_container_add (GTK_CONTAINER (window), area);
  gtk_widget_realize (area);
  gdk_window_set_back_pixmap (area->window, bg_pixmap, 0);

  gtk_widget_show (area);
  gtk_widget_show (window);
  gdk_window_clear (window->window);
  gdk_window_clear (area->window);
}

static void
sanalyzer_cleanup (void)
{
  if (window)
    {
      gtk_widget_destroy (window);
    }
  if (gc)
    {
      gdk_gc_unref (gc);
      gc = NULL;
    }
  if (bg_pixmap)
    {
      gdk_pixmap_unref (bg_pixmap);
      bg_pixmap = NULL;
    }
  if (draw_pixmap)
    {
      gdk_pixmap_unref (draw_pixmap);
      draw_pixmap = NULL;
    }
  if (bar)
    {
      gdk_pixmap_unref (bar);
      bar = NULL;
    }
}


static void
sanalyzer_playback_start (void)
{
  li_pcm = fopen ("/tmp/temp.pcm.links", "w");
  re_pcm = fopen ("/tmp/temp.pcm.rechts", "w");
  li_frq = fopen ("/tmp/temp.frq.links", "w");
  re_frq = fopen ("/tmp/temp.frq.rechts", "w");

  discrun = 1;
  discrun_pcm = 1;
  (void) gettimeofday (&startsong, NULL);
  (void) gettimeofday (&startsongF, NULL);

  if (window)
    {
      gdk_window_set_back_pixmap (area->window, draw_pixmap, 0);
      gdk_window_clear (area->window);
    }
}


static void
sanalyzer_playback_stop (void)
{
  int len;
  if (GTK_WIDGET_REALIZED (area))
    {
      gdk_window_set_back_pixmap (area->window, bg_pixmap, 0);
      gdk_window_clear (area->window);
    }
  (void) fclose (li_frq);
  (void) fclose (li_pcm);
  (void) fclose (re_frq);
  (void) fclose (re_pcm);
}


static void
sanalyzer_render_freq (gint16 data[2][256])
{
  gint i;
  timeval as;
  long now;

  i = gettimeofday (&as, NULL);
  now = as.tv_usec - startsongF.tv_usec;


  (void) fprintf (li_frq, "\nT%ldC%d\n", now, discrun);
  for (i = 0; i < 256; i++)
    (void) fprintf (li_frq, "%d/", data[0][i]);

  (void) fprintf (re_frq, "\nT%ldC%d\n", now, discrun);
  for (i = 0; i < 256; i++)
    (void) fprintf (re_frq, "%d/", data[1][i]);

  startsongF.tv_usec = as.tv_usec;
  discrun++;
}


static void
sanalyzer_render_pcm (gint16 data[2][512])
{
  gint i;
  timeval as;
  long now;

  i = gettimeofday (&as, NULL);
  now = as.tv_usec - startsong.tv_usec;

  (void) fprintf (li_pcm, "\nT%ldC%d\n", now, discrun_pcm);
  for (i = 0; i < 512; i++)
    (void) fprintf (li_pcm, "%d/", data[0][i]);

  (void) fprintf (re_pcm, "\nT%ldC%d\n", now, discrun_pcm);
  for (i = 0; i < 512; i++)
    (void) fprintf (re_pcm, "%d/", data[1][i]);

  startsong.tv_usec = as.tv_usec;
  discrun_pcm++;
}
